/*
 * ppplicznik -- counter of ppp connections with many features
 * Copyright (C) 2001 Artur Siekielski <gruesome@go2.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#pragma implementation

#include <ncurses.h>
#include <unistd.h>

#include <string>
#include <sstream>
#include <algorithm>
#include <numeric>

#include "koszty.h"
#include "ekran.h"
#include "main.h"
#include "loc.h"

using namespace std;

ekran::ekran()
{
	ustawMape();
	initscr();
	curs_set(0);
	timeout(0);
	noecho();
	start_color();
	init_pair(1, polaczenie::konf.kolory_napisy[0], polaczenie::konf.kolory_napisy[1]);
	init_pair(2, polaczenie::konf.kolory_dane[0], polaczenie::konf.kolory_dane[1]);
	init_pair(3, polaczenie::konf.kolory_wyk_od[0], polaczenie::konf.kolory_wyk_od[1]);
	init_pair(4, polaczenie::konf.kolory_ramka[0], polaczenie::konf.kolory_ramka[1]);
	init_pair(5, polaczenie::konf.kolory_wyk_wys[0], polaczenie::konf.kolory_wyk_wys[1]);
	init_pair(6, polaczenie::konf.kolory_limit[0], polaczenie::konf.kolory_limit[1]);
	const int nlines = polaczenie::konf.cowysw.size() * 2 + 1;
	const int ncolumns = 44;
	int ngraph;
	if (polaczenie::konf.wykres)
		ngraph = polaczenie::konf.wyk_max;
	else
		ngraph = 0;
	int check = nlines + polaczenie::konf.gorna_przerwa + ngraph + 2;
	if (check > LINES) {
		endwin();
		ostringstream os;
		os << gettext("Fatal: too small number of lines on terminal (is ") << LINES << ", " 
				<< check << gettext(" required)");
		throw error(os.str());
	}
	okno = newwin(nlines, ncolumns, polaczenie::konf.gorna_przerwa, COLS / 2 - (ncolumns / 2));
	box(okno, ACS_VLINE, ACS_HLINE);
	if (polaczenie::konf.kolory_ramka_bold) {
		wbkgd(okno, (' ' | COLOR_PAIR(4) | A_BOLD ));
		wattroff(okno, A_BOLD);
	}
	else {
		wattroff(okno, A_BOLD);
		wbkgd(okno, (' ' | COLOR_PAIR(4)));
	}
	wrefresh(okno);
}

void ekran::rysujWykres()
{
	int do_ilu = static_cast<int>((accumulate(polaczenie::pepepe.s.begin(), polaczenie::pepepe.s.end(), float()))
				/ polaczenie::pepepe.s.size() / polaczenie::konf.wyk_one);
	int do_ilu2 = static_cast<int>((accumulate(polaczenie::pepepe.s2.begin(), polaczenie::pepepe.s2.end(), float()))
				/ polaczenie::pepepe.s2.size() / polaczenie::konf.wyk_one);
	if (do_ilu > polaczenie::konf.wyk_max)
		do_ilu = polaczenie::konf.wyk_max;
	if (polaczenie::konf.kolory_wyk_od_bold)
		attron(COLOR_PAIR(3) | A_BOLD);
	else {
		attron(COLOR_PAIR(3));
		attroff(A_BOLD);
	}
	for (int i = 0; i < do_ilu; i++) {
		mvaddch(LINES - i, COLS - 1, ACS_VLINE);
	}
	if (polaczenie::konf.kolory_wyk_wys_bold)
		attron(COLOR_PAIR(5) | A_BOLD);
	else {
		attron(COLOR_PAIR(5));
		attroff(A_BOLD);
	}
	for (int i = 0; i < do_ilu2; i++) {
		mvaddch(LINES - i, COLS - 1, ACS_VLINE);
	}
	for (int i = 1; i < COLS; i++) {
		for (int j = 0; j < polaczenie::konf.wyk_max; j++) {
			move(LINES - j, i);
			chtype znak = inch();
			mvaddch(LINES - j, i - 1, znak);
		}
	}
	for (int i = 0; i < polaczenie::konf.wyk_max; i++) {
		mvaddch(LINES - i, COLS - 1, ' ');
	}
	touchwin(okno);
}		

inline void ekran::kolorNapisow()
{
	if (polaczenie::konf.kolory_napisy_bold)
		wattron(okno, COLOR_PAIR(1) | A_BOLD);
	else {
		wattron(okno, COLOR_PAIR(1));
		attroff(A_BOLD);
	}
}

inline void ekran::kolorDanych()
{
	if (polaczenie::konf.kolory_dane_bold)
		wattron(okno, COLOR_PAIR(2) | A_BOLD);
	else {
		attroff(A_BOLD);
		wattron(okno, COLOR_PAIR(2));
	}
}

inline void ekran::w1(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Connection time:"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%7d:%02d", connection.trwanie / 60, connection.trwanie % 60);
}

inline void ekran::w2(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Next impulse in:"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%7d:%02d", static_cast<int>(connection.do_nastepnego / 60),
			connection.do_nastepnego % 60);
}

inline void ekran::w3(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Cost:"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%10.2f", connection.koszt);
}

inline void ekran::w4(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Today's cost:"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%10.2f", polaczenie::konf.km_dzien);
}

inline void ekran::w5(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Monthly cost:"));
	kolorDanych();
	if (polaczenie::konf.hours > 0 && polaczenie::konf.km_mies < (polaczenie::konf.hours
				* polaczenie::konf.hour_cost))
		mvwprintw(okno, y, 27, "%4.2f (%4.2f)", polaczenie::konf.hours * polaczenie::konf.hour_cost, polaczenie::konf.km_mies);
	else
		mvwprintw(okno, y, 27, "%10.2f", polaczenie::konf.km_mies);
}

inline void ekran::w6(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Incoming bytes:"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%10d%3s", static_cast<int>(polaczenie::pepepe.akt_odebrane), "/s");
}

inline void ekran::w7(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Outcoming bytes:"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%10d%3s", static_cast<int>(polaczenie::pepepe.akt_wyslane), "/s");
}

inline void ekran::w8(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Average (incoming):"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%10d%3s", static_cast<int>(polaczenie::pepepe.sredni), "/s");
}

inline void ekran::w9(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Average (outcoming):"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%10d%3s", static_cast<int>(polaczenie::pepepe.sredni_wys), "/s");
}

inline void ekran::w10(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Total received:"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%8.1f%3s", polaczenie::pepepe.bajty_odebrane / 1024, "kB");
}

inline void ekran::w11(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("Total sent:"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%8.1f%3s", polaczenie::pepepe.bajty_wyslane / 1024, "kB");
}

inline void ekran::w12(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-25s", gettext("IP:"));
	kolorDanych();
	mvwprintw(okno, y, 27, "%-15s", polaczenie::konf.zmienna_ip.c_str());
}

inline void ekran::w13(const polaczenie& connection, int y)
{
	kolorNapisow();
	mvwprintw(okno, y, 2, "%-6s", gettext("Hostname:"));
	kolorDanych();
	mvwprintw(okno, y, 8, "%s", polaczenie::konf.zmienna_host.c_str());
}

void ekran::ustawMape()
{
	S[1]=&ekran::w1;
	S[2]=&ekran::w2;
	S[3]=&ekran::w3;
	S[4]=&ekran::w4;
	S[5]=&ekran::w5;
	S[6]=&ekran::w6;
	S[7]=&ekran::w7;
	S[8]=&ekran::w8;
	S[9]=&ekran::w9;
	S[10]=&ekran::w10;
	S[11]=&ekran::w11;
	S[12]=&ekran::w12;
	S[13]=&ekran::w13;
}

void ekran::wyswietl(const polaczenie& connection)
{
	int gdzie = 1;
	for (vector<int>::const_iterator it = polaczenie::konf.cowysw.begin();
			it != polaczenie::konf.cowysw.end(); ++it) {
		(this->*S[*it])(connection, gdzie);
		gdzie += 2;
	}
	if (polaczenie::konf.rozlacz) {
		if (polaczenie::konf.rozlacz_enabled) {
			attron(A_BLINK);
			attron(COLOR_PAIR(6));
			mvwprintw(stdscr, 2, COLS / 2 - 22, gettext("Disconnecting %d seconds before end of impulse"),
					polaczenie::konf.rozlacz_kiedy);
			attroff(A_BLINK);
		}
		else {
			move(2, 0);
			clrtoeol();
		}
		wrefresh(stdscr);
	}
	if (polaczenie::konf.wykres) {
		rysujWykres();
		wrefresh(stdscr);
	}
	if (polaczenie::konf.limit && polaczenie::konf.limit_ile < polaczenie::konf.km_dzien) {
		attron(A_BLINK);
		if (polaczenie::konf.kolory_limit_bold)
			attron(A_BOLD);
		else
			attroff(A_BOLD);
		attron(COLOR_PAIR(6));
		mvwprintw(stdscr, 1, COLS / 2 - 12, gettext("LIMIT OVERDRAFT"));
		attroff(A_BLINK);
	}
	wrefresh(okno);
}
				
				

ekran::~ekran()
{
	wclear(okno);
	wclear(stdscr);
	wrefresh(okno);
	wrefresh(stdscr);
	delwin(okno);
	endwin();
}

