/*
 * ppplicznik -- counter of ppp connections with many features
 * Copyright (C) 2001 Artur Siekielski <gruesome@go2.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#pragma implementation

#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <ncurses.h>

#include <fstream>
#include <iostream>
#include <iomanip>
#include <string>
#include <cstdlib>

#include "koszty.h"
#include "ekran.h"
#include "main.h"
#include "loc.h"


#define VERSION "1.0.0"

using namespace std;

bool koniec;
bool budzik;

void program::sprawdzLock()
{
	struct stat st;
	if (stat(polaczenie::konf.plik_lock.c_str(), &st) == 0) {
		zapisywac = false;
		cout << gettext("Warning: read-only session") << endl;
		ifstream p(plik_lock.c_str());
		if (!p)
			throw error(gettext("Cannot open lock file!"));
		int pid;
		p >> pid;
		p.close();
		cout << gettext("Blocked by pid ") << pid << endl;
		sleep(1);
		return;
	}
	else {
		zapisywac = true;
		ofstream p(plik_lock.c_str());
		p << getpid();
		if (!p)
			throw error(gettext("Cannot write lock file!"));
	}
}
	
program::program()
{
	zlicz = false;
	plik_logow = "/var/log/ppplicznik.log";
	const char* const termname = getenv("TERM");
	const string sterm = termname;
	if (sterm.find("linux") == string::npos && (termname == NULL || sterm.find("xterm") == string::npos))
		putenv("TERM=linux");
	setlocale(LC_MESSAGES, "");
	bindtextdomain("ppplicznik", "/usr/share/locale");
	textdomain("ppplicznik");
	cout.sync_with_stdio(false);
}

program::~program()
{
	if (zlicz) {
		delete ekranik;
		polaczenie::konf.km_koszt_ost_pol = aktualne.koszt;
		polaczenie::konf.km_wys_ost_pol = aktualne.pepepe.bajty_wyslane;
		polaczenie::konf.km_ode_ost_pol = aktualne.pepepe.bajty_odebrane;
		aktualne.konwertuj();
		if (zapisywac) {
			log_zapis.open(plik_logow.c_str(), ios::out | ios::ate | ios::app);
			log_zapis << aktualne;
			log_zapis.close();
			remove(plik_lock.c_str());
		}
		wypiszPodsumowanie();
	}
}

void program::wypiszPodsumowanie()
{
	cout.setf(ios::left);
	cout << gettext("Stats of the last connection:\n") << setw(50) << gettext("Cost:")
		<< setw(0) << setiosflags(ios::fixed) << setprecision(2)
		<< polaczenie::konf.km_koszt_ost_pol << '\n' << setw(50) << gettext("Kilobytes sent:")
		<< setw(0) << setiosflags(ios::fixed) << setprecision(1)
		<< polaczenie::konf.km_wys_ost_pol / 1024 << '\n' << setw(50) << gettext("Kilobytes received:")
		<< setw(0) << setiosflags(ios::fixed) << setprecision(1)
		<< polaczenie::konf.km_ode_ost_pol / 1024 << "\n\n";
	cout << setw(50) << gettext("Monthly cost:") << setw(0)
		<< setiosflags(ios::fixed) << setprecision(2)
		<< polaczenie::konf.km_mies << '\n' << setw(50) << gettext("Today's cost:")
		<< setw(0) << setiosflags(ios::fixed) << setprecision(2) << polaczenie::konf.km_dzien << '\n';
}

void program::sprawdzPolaczenie()
{
	if (!aktualne.pepepe.czyAktywny())
		throw error(gettext("There is no connection."));
}

inline bool program::czyDobraData(const polaczenie& p)
{
	switch (p.konf.wy) {
		case konfiguracja::miesiac:
			if (p.poczatek_tm.tm_mon != p.konf.akt.tm_mon ||
					p.poczatek_tm.tm_year != p.konf.akt.tm_year)
				return false;
			break;
		case konfiguracja::specific:
			if (polaczenie::konf.yspec == 0)
				polaczenie::konf.yspec = p.poczatek_tm.tm_year;
			if (p.poczatek_tm.tm_year != polaczenie::konf.yspec
					|| p.poczatek_tm.tm_mon != polaczenie::konf.mspec)
				return false;
			break;
		case konfiguracja::wszystko:
			return true;
	}
	return true;
}

void program::obliczKosztyMiesieczne()
{
	log_odczyt.open(plik_logow.c_str(), ios::in);
	if (!log_odczyt)
		return;
	polaczenie tymcz;
	float koszta = 0, koszt_dzien = 0;
	while (log_odczyt >> tymcz) {
		if (czyDobraData(tymcz) == false)
			continue;
		koszta += tymcz.koszt;
		if (tymcz.poczatek_tm.tm_yday == tymcz.konf.akt.tm_yday)
			koszt_dzien += tymcz.koszt;
	}
	polaczenie::konf.km_mies = koszta;
	polaczenie::konf.km_dzien = koszt_dzien;
	log_odczyt.close();
}

void program::wyswietlPlik()
{
	log_odczyt.open(plik_logow.c_str(), ios::in);
	if (!log_odczyt || log_odczyt.eof())
		return;
	polaczenie tymcz;
	int polaczenia = 0;
	int impulsy = 0;
	int czas = 0;
	float koszta = 0;
	double bw = 0, bo = 0;
	while (log_odczyt >> tymcz) {
		if (czyDobraData(tymcz) == false)
			continue;
		++polaczenia;
		impulsy += tymcz.impulsy;
		czas += tymcz.trwanie;
		koszta += tymcz.koszt;
		bw += tymcz.pepepe.bajty_wyslane;
		bo += tymcz.pepepe.bajty_odebrane;
		cout << tymcz;
	}
	log_odczyt.close();
	cout << '\n';
#if __GNUC__ < 3
	cout.form("%-30s%d\n", gettext("Number of connections:"), polaczenia);
	cout.form("%-30s%d\n", gettext("Number of impulses:"), impulsy);
	cout.form("%-30s%d:%02d:%02d\n", gettext("Total time:"), static_cast<int>(czas / 3600),
			static_cast<int>(czas % 3600 / 60), czas % 60);
	if (polaczenie::konf.hours > 0 && polaczenie::konf.hours * polaczenie::konf.hour_cost < koszta)
		cout.form("%-30s%.2f, (%.2f)\n", gettext("Total Cost:"), polaczenie::konf.hours * polaczenie::konf.hour_cost, koszta);
	else 
		cout.form("%-30s%.2f\n", gettext("Total Cost:"), koszta);
	cout.form("%-30s%-12.1f%-12.1f\n", gettext("Data sent and received in kB:"), bw / 1024, bo / 1024);
#else
	cout << setw(30) << left << gettext("Number of connections:") << polaczenia << '\n';
	cout << setw(30) << left << gettext("Number of impulses:") << impulsy << '\n';
	cout << setw(30) << left << gettext("Total time:") << static_cast<int>(czas/3600)
		<< setw(0) << ':' << setw(2) << setfill('0') << right
		<< static_cast<int>(czas % 3600 / 60) << setw(0) << ':' << setw(2) << setfill('0')
		<< czas % 60 << '\n';
	if (polaczenie::konf.hours > 0 && (polaczenie::konf.hours * polaczenie::konf.hour_cost) > koszta)
		cout << setfill(' ') << setw(30) << left << gettext("Total Cost:")
			<< setprecision(2) << setiosflags(ios::fixed) << (polaczenie::konf.hours * polaczenie::konf.hour_cost) << ", (" << koszta << ')' << '\n';
	else
		cout << setfill(' ') << setw(30) << left << gettext("Total Cost:")
			<< setprecision(2) << setiosflags(ios::fixed) << koszta << '\n';
	cout << setw(30) << left << gettext("Data sent and received in kB:") << setw(12)
		<< setprecision(1) << bw / 1024 << setw(12) << setprecision(1) << bo / 1024 << '\n';
#endif
}

void program::printHelp()
{
	cout << "ppplicznik-" VERSION ", copyright (C) 2001 Artur Siekielski <gruesome@go2.pl>" << '\n';
	cout << gettext("Usage: ppplicznik [options]") << '\n';
	cout << gettext("Options:") << '\n';
	cout << gettext("-n	counts ppp connection without checking activity\n-a	show info about all logged connections\n-m	show info about connections from the current month\n-M month -Y year  show connections from specified month and year\n-k file	configuration file\n-l file	file with logged connections\n-h	shows short help (you're reading it at the moment)\n\nFor more info read ppplicznik's manual page\n\n");
}

void program::zliczaj()
{
	zlicz = true;
	sprawdzLock();
	polaczenie::konf.czytajZmienne();
	obliczKosztyMiesieczne();
	aktualne.poczatek = aktualne.koniec = time(0);
	if (polaczenie::konf.stt)
		aktualne.poczatek = polaczenie::konf.t_mod;
	aktualne.poczatek -= polaczenie::konf.doliczane;
	ekranik = new ekran;
	char znak;
	if (polaczenie::konf.rozlacz)
		touchwin(ekranik->okno);
	while (true) {
		alarm(1);
		if (koniec)
			throw error(gettext("Killed by signal."));
		aktualne.koniec++;
		aktualne.oblicz();
		aktualne.pepepe.wezStatystyki();
		ekranik->wyswietl(aktualne);
		if (polaczenie::konf.rozlacz) {
			timeout(0);
			znak = wgetch(stdscr);
			if (znak == 'r')
				polaczenie::konf.rozlacz_enabled = !polaczenie::konf.rozlacz_enabled;
		}
		sleep(10);
	}
}

int program::czytajOpcje(int argc, char* argv[])
{
	int opt;
	bool wysw = false;
	polaczenie::konf.yspec = 0;
	while ((opt = getopt(argc, argv, "k:l:M:Y:wanmh")) != -1) {
		switch (opt) {
			case 'a':
				polaczenie::konf.wy = konfiguracja::wszystko;
				wysw = true;
				break;
			case 'm':
				polaczenie::konf.wy = konfiguracja::miesiac;
				wysw = true;
				break;
			case 'n':
				polaczenie::konf.bez_sprawdzenia = true;
				break;
			case 'l':
				plik_logow = optarg;
				break;
			case 'M':
				polaczenie::konf.wy = konfiguracja::specific;
				polaczenie::konf.mspec = atoi(optarg) - 1;
				wysw = true;
				break;
			case 'Y':
				polaczenie::konf.wy = konfiguracja::specific;
				polaczenie::konf.yspec = atoi(optarg) - 1900;
				wysw = true;
				break;
			case 'w':
				wysw = true;
				break;
			case 'k':
				polaczenie::konf.plik_konf = optarg;
				break;
			case 'h':
				printHelp();
				return 0;
			case ':':
				printHelp();
				return 1;
			case '?':
				printHelp();
				return 1;
			default:
				printHelp();
				return 1;
		}
	}
	polaczenie::konf.parsujPlik();
	plik_lock = polaczenie::konf.plik_lock;
	polaczenie::konf.ustawReguleNaTeraz();
	if (wysw) {
		wyswietlPlik();
		return 0;
	}
	else {
		probujLiczyc();
	}
	return 0;
}

void program::probujLiczyc()
{
	if (polaczenie::konf.bez_sprawdzenia == false)
		sprawdzPolaczenie();
	zliczaj();
}

void lap(int sig)
{
	koniec = true;

}

void dz(int sig)
{
	int status, pid;
	pid = waitpid(WAIT_ANY, &status, WNOHANG);
}

void al(int sig)
{
	budzik = true;
}

program moj;

int main(int argc, char *argv[])
{
	signal(SIGINT, lap);
	signal(SIGTERM, lap);
	signal(SIGCHLD, dz);
	signal(SIGALRM, al);
	try {
		return moj.czytajOpcje(argc, argv);
	}
	catch (const error& e) {
		cerr << e.what() << endl;
		return 1;
	}
}
